=== modified file 'NEWS'
--- NEWS	2006-07-11 15:17:03 +0000
+++ NEWS	2006-07-12 02:27:25 +0000
@@ -113,6 +113,9 @@
     * File-ids containing single- or double-quotes are handled correctly by
       push.  (#52227, Aaron Bentley)
 
+    * Normalize unicode filenames to ensure cross-platform consistency.
+      (John Arbash Meinel, #43689)
+
     * Bundles can properly read binary files with a plain '\r' in them.
       (John Arbash Meinel, #51927)
 

=== modified file 'bzrlib/add.py'
--- bzrlib/add.py	2006-06-20 04:45:30 +0000
+++ bzrlib/add.py	2006-07-02 05:28:12 +0000
@@ -266,9 +266,7 @@
                     ignore_glob = tree.is_ignored(subp)
                     if ignore_glob is not None:
                         # mutter("skip ignored sub-file %r", subp)
-                        if ignore_glob not in ignored:
-                            ignored[ignore_glob] = []
-                        ignored[ignore_glob].append(subp)
+                        ignored.setdefault(ignore_glob, []).append(subp)
                     else:
                         #mutter("queue to add sub-file %r", subp)
                         dirs_to_add.append((FastPath(subp, subf), this_ie))

=== modified file 'bzrlib/builtins.py'
--- bzrlib/builtins.py	2006-07-04 15:04:11 +0000
+++ bzrlib/builtins.py	2006-07-09 20:12:49 +0000
@@ -375,6 +375,9 @@
     encoding_type = 'replace'
 
     def run(self, names_list):
+        if names_list is None:
+            names_list = []
+
         if len(names_list) < 2:
             raise BzrCommandError("missing file argument")
         tree, rel_names = tree_files(names_list)

=== modified file 'bzrlib/errors.py'
--- bzrlib/errors.py	2006-07-10 02:16:27 +0000
+++ bzrlib/errors.py	2006-07-11 15:05:36 +0000
@@ -1,15 +1,15 @@
 # Copyright (C) 2005, 2006 Canonical
-
+#
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
 # the Free Software Foundation; either version 2 of the License, or
 # (at your option) any later version.
-
+#
 # This program is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 # GNU General Public License for more details.
-
+#
 # You should have received a copy of the GNU General Public License
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
@@ -280,6 +280,10 @@
             self.extra = ''
 
 
+class InvalidNormalization(PathError):
+    """Path %(path)r is not unicode normalized"""
+
+
 # TODO: This is given a URL; we try to unescape it but doing that from inside
 # the exception object is a bit undesirable.
 # TODO: Probably this behavior of should be a common superclass 

=== modified file 'bzrlib/inventory.py'
--- bzrlib/inventory.py	2006-06-26 19:06:55 +0000
+++ bzrlib/inventory.py	2006-07-02 05:45:50 +0000
@@ -36,6 +36,7 @@
 import types
 
 import bzrlib
+from bzrlib import errors, osutils
 from bzrlib.osutils import (pumpfile, quotefn, splitpath, joinpath,
                             pathjoin, sha_strings)
 from bzrlib.errors import (NotVersionedError, InvalidEntryName,
@@ -1061,7 +1062,7 @@
 
         Returns the new entry object."""
         
-        parts = bzrlib.osutils.splitpath(relpath)
+        parts = osutils.splitpath(relpath)
 
         if len(parts) == 0:
             if file_id is None:
@@ -1237,6 +1238,16 @@
     """
     if file_id is None:
         file_id = bzrlib.workingtree.gen_file_id(name)
+
+    norm_name, can_access = osutils.normalized_filename(name)
+    if norm_name != name:
+        if can_access:
+            name = norm_name
+        else:
+            # TODO: jam 20060701 This would probably be more useful
+            #       if the error was raised with the full path
+            raise errors.InvalidNormalization(name)
+
     if kind == 'directory':
         return InventoryDirectory(file_id, name, parent_id)
     elif kind == 'file':
@@ -1247,7 +1258,6 @@
         raise BzrError("unknown kind %r" % kind)
 
 
-
 _NAME_RE = None
 
 def is_valid_name(name):

=== modified file 'bzrlib/osutils.py'
--- bzrlib/osutils.py	2006-06-30 22:42:39 +0000
+++ bzrlib/osutils.py	2006-07-03 18:51:47 +0000
@@ -264,14 +264,19 @@
     try:
         fancy_rename(old, new, rename_func=os.rename, unlink_func=os.unlink)
     except OSError, e:
-        if e.errno in (errno.EPERM, errno.EACCES, errno.EBUSY):
-            # If we try to rename a non-existant file onto cwd, we get EPERM
-            # instead of ENOENT, this will raise ENOENT if the old path
-            # doesn't exist
+        if e.errno in (errno.EPERM, errno.EACCES, errno.EBUSY, errno.EINVAL):
+            # If we try to rename a non-existant file onto cwd, we get 
+            # EPERM or EACCES instead of ENOENT, this will raise ENOENT 
+            # if the old path doesn't exist, sometimes we get EACCES
+            # On Linux, we seem to get EBUSY, on Mac we get EINVAL
             os.lstat(old)
         raise
 
 
+def _mac_getcwd():
+    return unicodedata.normalize('NFKC', os.getcwdu())
+
+
 # Default is to just use the python builtins, but these can be rebound on
 # particular platforms.
 abspath = _posix_abspath
@@ -315,6 +320,8 @@
     def rmtree(path, ignore_errors=False, onerror=_win32_delete_readonly):
         """Replacer for shutil.rmtree: could remove readonly dirs/files"""
         return shutil.rmtree(path, ignore_errors, onerror)
+elif sys.platform == 'darwin':
+    getcwd = _mac_getcwd
 
 
 def get_terminal_encoding():
@@ -788,39 +795,35 @@
     return _platform_normalizes_filenames
 
 
+def _accessible_normalized_filename(path):
+    """Get the unicode normalized path, and if you can access the file.
+
+    On platforms where the system normalizes filenames (Mac OSX),
+    you can access a file by any path which will normalize correctly.
+    On platforms where the system does not normalize filenames 
+    (Windows, Linux), you have to access a file by its exact path.
+
+    Internally, bzr only supports NFC/NFKC normalization, since that is 
+    the standard for XML documents.
+
+    So return the normalized path, and a flag indicating if the file
+    can be accessed by that path.
+    """
+
+    return unicodedata.normalize('NFKC', unicode(path)), True
+
+
+def _inaccessible_normalized_filename(path):
+    __doc__ = _accessible_normalized_filename.__doc__
+
+    normalized = unicodedata.normalize('NFKC', unicode(path))
+    return normalized, normalized == path
+
+
 if _platform_normalizes_filenames:
-    def unicode_filename(path):
-        """Make sure 'path' is a properly normalized filename.
-
-        On platforms where the system normalizes filenames (Mac OSX),
-        you can access a file by any path which will normalize
-        correctly.
-        Internally, bzr only supports NFC/NFKC normalization, since
-        that is the standard for XML documents.
-        So we return an normalized path, and indicate this has been
-        properly normalized.
-
-        :return: (path, is_normalized) Return a path which can
-                access the file, and whether or not this path is
-                normalized.
-        """
-        return unicodedata.normalize('NFKC', path), True
+    normalized_filename = _accessible_normalized_filename
 else:
-    def unicode_filename(path):
-        """Make sure 'path' is a properly normalized filename.
-
-        On platforms where the system does not normalize filenames 
-        (Windows, Linux), you have to access a file by its exact path.
-        Internally, bzr only supports NFC/NFKC normalization, since
-        that is the standard for XML documents.
-        So we return the original path, and indicate if this is
-        properly normalized.
-
-        :return: (path, is_normalized) Return a path which can
-                access the file, and whether or not this path is
-                normalized.
-        """
-        return path, unicodedata.normalize('NFKC', path) == path
+    normalized_filename = _inaccessible_normalized_filename
 
 
 def terminal_width():

=== modified file 'bzrlib/tests/blackbox/__init__.py'
--- bzrlib/tests/blackbox/__init__.py	2006-07-04 15:04:11 +0000
+++ bzrlib/tests/blackbox/__init__.py	2006-07-09 20:12:49 +0000
@@ -64,6 +64,7 @@
                      'bzrlib.tests.blackbox.test_logformats',
                      'bzrlib.tests.blackbox.test_merge',
                      'bzrlib.tests.blackbox.test_missing',
+                     'bzrlib.tests.blackbox.test_mv',
                      'bzrlib.tests.blackbox.test_outside_wt',
                      'bzrlib.tests.blackbox.test_pull',
                      'bzrlib.tests.blackbox.test_push',

=== added file 'bzrlib/tests/blackbox/test_mv.py'
--- bzrlib/tests/blackbox/test_mv.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/blackbox/test_mv.py	2006-07-11 01:14:17 +0000
@@ -0,0 +1,130 @@
+# Copyright (C) 2006 by Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""Test for 'bzr mv'"""
+
+import os
+
+from bzrlib.tests import TestCaseWithTransport
+
+
+class TestMove(TestCaseWithTransport):
+
+    def test_mv_modes(self):
+        """Test two modes of operation for mv"""
+        tree = self.make_branch_and_tree('.')
+        files = self.build_tree(['a', 'c', 'subdir/'])
+        tree.add(['a', 'c', 'subdir'])
+
+        self.run_bzr('mv', 'a', 'b')
+        self.failUnlessExists('b')
+        self.failIfExists('a')
+
+        self.run_bzr('mv', 'b', 'subdir')
+        self.failUnlessExists('subdir/b')
+        self.failIfExists('b')
+
+        self.run_bzr('mv', 'subdir/b', 'a')
+        self.failUnlessExists('a')
+        self.failIfExists('subdir/b')
+
+        self.run_bzr('mv', 'a', 'c', 'subdir')
+        self.failUnlessExists('subdir/a')
+        self.failUnlessExists('subdir/c')
+        self.failIfExists('a')
+        self.failIfExists('c')
+
+        self.run_bzr('mv', 'subdir/a', 'subdir/newa')
+        self.failUnlessExists('subdir/newa')
+        self.failIfExists('subdir/a')
+
+    def test_mv_unversioned(self):
+        self.build_tree(['unversioned.txt'])
+        self.run_bzr_error(
+            ["^bzr: ERROR: can't rename: old name .* is not versioned$"],
+            'mv', 'unversioned.txt', 'elsewhere')
+
+    def test_mv_nonexisting(self):
+        self.run_bzr_error(
+            ["^bzr: ERROR: can't rename: old working file .* does not exist$"],
+            'mv', 'doesnotexist', 'somewhereelse')
+
+    def test_mv_unqualified(self):
+        self.run_bzr_error(['^bzr: ERROR: missing file argument$'], 'mv')
+        
+    def test_mv_invalid(self):
+        tree = self.make_branch_and_tree('.')
+        self.build_tree(['test.txt', 'sub1/'])
+        tree.add(['test.txt'])
+
+        self.run_bzr_error(
+            ["^bzr: ERROR: destination u'sub1' is not a versioned directory$"],
+            'mv', 'test.txt', 'sub1')
+        
+        self.run_bzr_error(
+            ["^bzr: ERROR: can't determine destination directory id for u'sub1'$"],
+            'mv', 'test.txt', 'sub1/hello.txt')
+        
+    def test_mv_dirs(self):
+        tree = self.make_branch_and_tree('.')
+        self.build_tree(['hello.txt', 'sub1/'])
+        tree.add(['hello.txt', 'sub1'])
+
+        self.run_bzr('mv', 'sub1', 'sub2')
+        self.failUnlessExists('sub2')
+        self.failIfExists('sub1')
+        self.run_bzr('mv', 'hello.txt', 'sub2')
+        self.failUnlessExists("sub2/hello.txt")
+        self.failIfExists("hello.txt")
+
+        tree.read_working_inventory()
+        tree.commit('commit with some things moved to subdirs')
+
+        self.build_tree(['sub1/'])
+        tree.add(['sub1'])
+        self.run_bzr('mv', 'sub2/hello.txt', 'sub1')
+        self.failIfExists('sub2/hello.txt')
+        self.failUnlessExists('sub1/hello.txt')
+        self.run_bzr('mv', 'sub2', 'sub1')
+        self.failIfExists('sub2')
+        self.failUnlessExists('sub1/sub2')
+
+    def test_mv_relative(self):
+        self.build_tree(['sub1/', 'sub1/sub2/', 'sub1/hello.txt'])
+        tree = self.make_branch_and_tree('.')
+        tree.add(['sub1', 'sub1/sub2', 'sub1/hello.txt'])
+        tree.commit('initial tree')
+
+        os.chdir('sub1/sub2')
+        self.run_bzr('mv', '../hello.txt', '.')
+        self.failUnlessExists('./hello.txt')
+        tree.read_working_inventory()
+        tree.commit('move to parent directory')
+
+        os.chdir('..')
+
+        self.run_bzr('mv', 'sub2/hello.txt', '.')
+        self.failUnlessExists('hello.txt')
+
+    def test_mv_smoke_aliases(self):
+        # just test that aliases for mv exist, if their behaviour is changed in
+        # the future, then extend the tests.
+        self.build_tree(['a'])
+        tree = self.make_branch_and_tree('.')
+        tree.add(['a'])
+
+        self.run_bzr('move', 'a', 'b')
+        self.run_bzr('rename', 'b', 'a')

=== modified file 'bzrlib/tests/blackbox/test_non_ascii.py'
--- bzrlib/tests/blackbox/test_non_ascii.py	2006-07-03 17:22:41 +0000
+++ bzrlib/tests/blackbox/test_non_ascii.py	2006-07-11 15:05:36 +0000
@@ -447,11 +447,11 @@
         bzr = self.run_bzr_decode
 
         txt = bzr('ls')
-        self.assertEqual(['a', 'b', self.info['filename']],
-                         txt.splitlines())
+        self.assertEqual(sorted(['a', 'b', self.info['filename']]),
+                         sorted(txt.splitlines()))
         txt = bzr('ls', '--null')
-        self.assertEqual(['a', 'b', self.info['filename'], ''],
-                         txt.split('\0'))
+        self.assertEqual(sorted(['', 'a', 'b', self.info['filename']]),
+                         sorted(txt.split('\0')))
 
         txt = bzr('ls', encoding='ascii', retcode=3)
         txt = bzr('ls', '--null', encoding='ascii', retcode=3)

=== modified file 'bzrlib/tests/blackbox/test_too_much.py'
--- bzrlib/tests/blackbox/test_too_much.py	2006-07-04 15:04:11 +0000
+++ bzrlib/tests/blackbox/test_too_much.py	2006-07-09 20:12:49 +0000
@@ -147,17 +147,6 @@
         self.runbzr('revert')
         os.chdir('..')
 
-    def test_mv_modes(self):
-        """Test two modes of operation for mv"""
-        self.runbzr('init')
-        self.build_tree(['a', 'c', 'subdir/'])
-        self.run_bzr_captured(['add', self.test_dir])
-        self.run_bzr_captured(['mv', 'a', 'b'])
-        self.run_bzr_captured(['mv', 'b', 'subdir'])
-        self.run_bzr_captured(['mv', 'subdir/b', 'a'])
-        self.run_bzr_captured(['mv', 'a', 'c', 'subdir'])
-        self.run_bzr_captured(['mv', 'subdir/a', 'subdir/newa'])
-
     def test_main_version(self):
         """Check output from version command and master option is reasonable"""
         # output is intentionally passed through to stdout so that we
@@ -677,81 +666,20 @@
         out = capture("help ci")
         out.index('aliases: ')
 
-        progress("can't rename unversioned file")
-        runbzr("rename test.txt new-test.txt", 3)
-
-        progress("adding a file")
-
-        runbzr("add test.txt")
-        self.assertEquals(capture("unknowns"), '')
-
-        progress("rename newly-added file")
-        runbzr("rename test.txt hello.txt")
-        self.assert_(os.path.exists("hello.txt"))
-        self.assert_(not os.path.exists("test.txt"))
-
-        self.assertEquals(capture("revno"), '0\n')
-
-        progress("add first revision")
-        runbzr(['commit', '-m', 'add first revision'])
-
-        progress("more complex renames")
-        os.mkdir("sub1")
-        runbzr("rename hello.txt sub1", 3)
-        runbzr("rename hello.txt sub1/hello.txt", 3)
-        runbzr("move hello.txt sub1", 3)
-
-        runbzr("add sub1")
-        runbzr("rename sub1 sub2")
-        runbzr("move hello.txt sub2")
-        self.assertEqual(capture("relpath sub2/hello.txt"),
-                         pathjoin("sub2", "hello.txt\n"))
-
-        self.assert_(exists("sub2"))
-        self.assert_(exists("sub2/hello.txt"))
-        self.assert_(not exists("sub1"))
-        self.assert_(not exists("hello.txt"))
-
-        runbzr(['commit', '-m', 'commit with some things moved to subdirs'])
-
-        mkdir("sub1")
-        runbzr('add sub1')
-        runbzr('move sub2/hello.txt sub1')
-        self.assert_(not exists('sub2/hello.txt'))
-        self.assert_(exists('sub1/hello.txt'))
-        runbzr('move sub2 sub1')
-        self.assert_(not exists('sub2'))
-        self.assert_(exists('sub1/sub2'))
-
-        runbzr(['commit', '-m', 'rename nested subdirectories'])
-
-        chdir('sub1/sub2')
-        self.assertEquals(capture('root')[:-1],
-                          pathjoin(self.test_dir, 'branch1'))
-        runbzr('move ../hello.txt .')
-        self.assert_(exists('./hello.txt'))
-        self.assertEquals(capture('relpath hello.txt'),
-                          pathjoin('sub1', 'sub2', 'hello.txt') + '\n')
-        self.assertEquals(capture('relpath ../../sub1/sub2/hello.txt'), pathjoin('sub1', 'sub2', 'hello.txt\n'))
-        runbzr(['commit', '-m', 'move to parent directory'])
-        chdir('..')
-        self.assertEquals(capture('relpath sub2/hello.txt'), pathjoin('sub1', 'sub2', 'hello.txt\n'))
-
-        runbzr('move sub2/hello.txt .')
-        self.assert_(exists('hello.txt'))
-
         f = file('hello.txt', 'wt')
         f.write('some nice new content\n')
         f.close()
 
+        runbzr("add hello.txt")
+        
         f = file('msg.tmp', 'wt')
         f.write('this is my new commit\nand it has multiple lines, for fun')
         f.close()
 
         runbzr('commit -F msg.tmp')
 
-        self.assertEquals(capture('revno'), '5\n')
-        runbzr('export -r 5 export-5.tmp')
+        self.assertEquals(capture('revno'), '1\n')
+        runbzr('export -r 1 export-1.tmp')
         runbzr('export export.tmp')
 
         runbzr('log')

=== modified file 'bzrlib/tests/test_inv.py'
--- bzrlib/tests/test_inv.py	2006-06-28 16:34:04 +0000
+++ bzrlib/tests/test_inv.py	2006-07-02 05:45:50 +0000
@@ -18,12 +18,11 @@
 import os
 import time
 
+from bzrlib import errors, inventory, osutils
 from bzrlib.branch import Branch
-import bzrlib.errors as errors
 from bzrlib.diff import internal_diff
 from bzrlib.inventory import (Inventory, ROOT_ID, InventoryFile,
     InventoryDirectory, InventoryEntry)
-import bzrlib.inventory as inventory
 from bzrlib.osutils import (has_symlinks, rename, pathjoin, is_inside_any, 
     is_inside_or_parent_of_any)
 from bzrlib.tests import TestCase, TestCaseWithTransport
@@ -200,6 +199,22 @@
         self.assertIsInstance(inventory.make_entry("directory", "name", ROOT_ID),
             inventory.InventoryDirectory)
 
+    def test_make_entry_non_normalized(self):
+        orig_normalized_filename = osutils.normalized_filename
+
+        try:
+            osutils.normalized_filename = osutils._accessible_normalized_filename
+            entry = inventory.make_entry("file", u'a\u030a', ROOT_ID)
+            self.assertEqual(u'\xe5', entry.name)
+            self.assertIsInstance(entry, inventory.InventoryFile)
+
+            osutils.normalized_filename = osutils._inaccessible_normalized_filename
+            self.assertRaises(errors.InvalidNormalization,
+                    inventory.make_entry, 'file', u'a\u030a', ROOT_ID)
+        finally:
+            osutils.normalized_filename = orig_normalized_filename
+
+
 class TestEntryDiffing(TestCaseWithTransport):
 
     def setUp(self):

=== modified file 'bzrlib/tests/test_nonascii.py'
--- bzrlib/tests/test_nonascii.py	2006-07-03 19:27:26 +0000
+++ bzrlib/tests/test_nonascii.py	2006-07-11 15:05:36 +0000
@@ -20,8 +20,9 @@
 import sys
 from unicodedata import normalize
 
-from bzrlib.osutils import pathjoin, normalizes_filenames, unicode_filename
-from bzrlib.tests import TestCaseWithTransport, TestSkipped
+from bzrlib import osutils
+from bzrlib.osutils import pathjoin
+from bzrlib.tests import TestCase, TestCaseWithTransport, TestSkipped
 from bzrlib.workingtree import WorkingTree
 
 
@@ -40,25 +41,59 @@
 
 
 a_circle_c = u'\xe5'
+a_circle_d = u'a\u030a'
 a_dots_c = u'\xe4'
-a_circle_d = u'a\u030a'
 a_dots_d = u'a\u0308'
 z_umlat_c = u'\u017d'
 z_umlat_d = u'Z\u030c'
 
 
-class UnicodeFilename(TestCaseWithTransport):
-    """Test that UnicodeFilename returns the expected values."""
+class TestNormalization(TestCase):
+    """Verify that we have our normalizations correct."""
 
-    def test_a_circle(self):
+    def test_normalize(self):
         self.assertEqual(a_circle_d, normalize('NFKD', a_circle_c))
         self.assertEqual(a_circle_c, normalize('NFKC', a_circle_d))
-
-        self.assertEqual((a_circle_c, True), unicode_filename(a_circle_c))
-        if normalizes_filenames():
-            self.assertEqual((a_circle_c, True), unicode_filename(a_circle_d))
+        self.assertEqual(a_dots_d, normalize('NFKD', a_dots_c))
+        self.assertEqual(a_dots_c, normalize('NFKC', a_dots_d))
+        self.assertEqual(z_umlat_d, normalize('NFKD', z_umlat_c))
+        self.assertEqual(z_umlat_c, normalize('NFKC', z_umlat_d))
+
+
+class NormalizedFilename(TestCaseWithTransport):
+    """Test normalized_filename and associated helpers"""
+
+    def test__accessible_normalized_filename(self):
+        anf = osutils._accessible_normalized_filename
+        # normalized_filename should allow plain ascii strings
+        # not just unicode strings
+        self.assertEqual((u'ascii', True), anf('ascii'))
+        self.assertEqual((a_circle_c, True), anf(a_circle_c))
+        self.assertEqual((a_circle_c, True), anf(a_circle_d))
+        self.assertEqual((a_dots_c, True), anf(a_dots_c))
+        self.assertEqual((a_dots_c, True), anf(a_dots_d))
+        self.assertEqual((z_umlat_c, True), anf(z_umlat_c))
+        self.assertEqual((z_umlat_c, True), anf(z_umlat_d))
+
+    def test__inaccessible_normalized_filename(self):
+        inf = osutils._inaccessible_normalized_filename
+        # normalized_filename should allow plain ascii strings
+        # not just unicode strings
+        self.assertEqual((u'ascii', True), inf('ascii'))
+        self.assertEqual((a_circle_c, True), inf(a_circle_c))
+        self.assertEqual((a_circle_c, False), inf(a_circle_d))
+        self.assertEqual((a_dots_c, True), inf(a_dots_c))
+        self.assertEqual((a_dots_c, False), inf(a_dots_d))
+        self.assertEqual((z_umlat_c, True), inf(z_umlat_c))
+        self.assertEqual((z_umlat_c, False), inf(z_umlat_d))
+
+    def test_functions(self):
+        if osutils.normalizes_filenames():
+            self.assertEqual(osutils.normalized_filename,
+                             osutils._accessible_normalized_filename)
         else:
-            self.assertEqual((a_circle_d, False), unicode_filename(a_circle_d))
+            self.assertEqual(osutils.normalized_filename,
+                             osutils._inaccessible_normalized_filename)
 
     def test_platform(self):
         # With FAT32 and certain encodings on win32
@@ -79,9 +114,9 @@
         present = sorted(os.listdir(u'.'))
         self.assertEqual(expected, present)
 
-    def test_access(self):
-        # We should always be able to access files by the path returned
-        # from unicode_filename
+    def test_access_normalized(self):
+        # We should always be able to access files created with 
+        # normalized filenames
         # With FAT32 and certain encodings on win32
         # a_circle_c and a_dots_c actually map to the same file
         # adding a suffix kicks in the 'preserving but insensitive'
@@ -93,9 +128,13 @@
             raise TestSkipped("filesystem cannot create unicode files")
 
         for fname in files:
-            path = unicode_filename(fname)[0]
             # We should get an exception if we can't open the file at
             # this location.
+            path, can_access = osutils.normalized_filename(fname)
+
+            self.assertEqual(path, fname)
+            self.assertTrue(can_access)
+
             f = open(path, 'rb')
             try:
                 # Check the contents
@@ -108,4 +147,32 @@
                              'contents of %s is incorrect: %r != %r'
                              % (path, shouldbe, actual))
 
-
+    def test_access_non_normalized(self):
+        # Sometimes we can access non-normalized files by their normalized
+        # path, verify that normalized_filename returns the right info
+        files = [a_circle_d+'.1', a_dots_d+'.2', z_umlat_d+'.3']
+
+        try:
+            self.build_tree(files)
+        except UnicodeError:
+            raise TestSkipped("filesystem cannot create unicode files")
+
+        for fname in files:
+            # We should get an exception if we can't open the file at
+            # this location.
+            path, can_access = osutils.normalized_filename(fname)
+
+            self.assertNotEqual(path, fname)
+
+            # We should always be able to access them from the name
+            # they were created with
+            f = open(fname, 'rb')
+            f.close()
+
+            # And normalized_filename sholud tell us correctly if we can
+            # access them by an alternate name
+            if can_access:
+                f = open(path, 'rb')
+                f.close()
+            else:
+                self.assertRaises(IOError, open, path, 'rb')

=== modified file 'bzrlib/tests/test_osutils.py'
--- bzrlib/tests/test_osutils.py	2006-06-29 20:26:43 +0000
+++ bzrlib/tests/test_osutils.py	2006-07-03 16:19:30 +0000
@@ -198,16 +198,16 @@
     def test_getcwd(self):
         # Make sure getcwd can handle unicode filenames
         try:
-            os.mkdir(u'B\xe5gfors')
+            os.mkdir(u'mu-\xb5')
         except UnicodeError:
             raise TestSkipped("Unable to create Unicode filename")
 
-        os.chdir(u'B\xe5gfors')
+        os.chdir(u'mu-\xb5')
         # TODO: jam 20060427 This will probably fail on Mac OSX because
         #       it will change the normalization of B\xe5gfors
         #       Consider using a different unicode character, or make
         #       osutils.getcwd() renormalize the path.
-        self.assertTrue(osutils._win32_getcwd().endswith(u'/B\xe5gfors'))
+        self.assertEndsWith(osutils._win32_getcwd(), u'mu-\xb5')
 
     def test_mkdtemp(self):
         tmpdir = osutils._win32_mkdtemp(dir='.')
@@ -257,6 +257,30 @@
             self.assertEqual(errno.ENOENT, e.errno)
 
 
+class TestMacFuncsDirs(TestCaseInTempDir):
+    """Test mac special functions that require directories."""
+
+    def test_getcwd(self):
+        # On Mac, this will actually create Ba\u030agfors
+        # but chdir will still work, because it accepts both paths
+        try:
+            os.mkdir(u'B\xe5gfors')
+        except UnicodeError:
+            raise TestSkipped("Unable to create Unicode filename")
+
+        os.chdir(u'B\xe5gfors')
+        self.assertEndsWith(osutils._mac_getcwd(), u'B\xe5gfors')
+
+    def test_getcwd_nonnorm(self):
+        # Test that _mac_getcwd() will normalize this path
+        try:
+            os.mkdir(u'Ba\u030agfors')
+        except UnicodeError:
+            raise TestSkipped("Unable to create Unicode filename")
+
+        os.chdir(u'Ba\u030agfors')
+        self.assertEndsWith(osutils._mac_getcwd(), u'B\xe5gfors')
+
 class TestSplitLines(TestCase):
 
     def test_split_unicode(self):

=== modified file 'bzrlib/tests/test_smart_add.py'
--- bzrlib/tests/test_smart_add.py	2006-07-03 04:59:44 +0000
+++ bzrlib/tests/test_smart_add.py	2006-07-11 18:34:09 +0000
@@ -1,12 +1,14 @@
 import os
 import unittest
 
+from bzrlib import errors, osutils
 from bzrlib.add import smart_add, smart_add_tree
-from bzrlib.tests import TestCaseWithTransport, TestCase
+from bzrlib.tests import TestCase, TestCaseWithTransport, TestSkipped
 from bzrlib.errors import NoSuchFile
 from bzrlib.inventory import InventoryFile, Inventory
 from bzrlib.workingtree import WorkingTree
 
+
 class TestSmartAdd(TestCaseWithTransport):
 
     def test_add_dot_from_root(self):
@@ -15,7 +17,6 @@
         paths = ("original/", "original/file1", "original/file2")
         self.build_tree(paths)
         wt = self.make_branch_and_tree('.')
-        branch = wt.branch
         smart_add_tree(wt, (u".",))
         for path in paths:
             self.assertNotEqual(wt.path2id(path), None)
@@ -26,7 +27,6 @@
         paths = ("original/", "original/file1", "original/file2")
         self.build_tree(paths)
         wt = self.make_branch_and_tree('.')
-        branch = wt.branch
         os.chdir("original")
         smart_add_tree(wt, (u".",))
         for path in paths:
@@ -40,7 +40,6 @@
                         "branch/original/file2")
         self.build_tree(branch_paths)
         wt = self.make_branch_and_tree('branch')
-        branch = wt.branch
         smart_add_tree(wt, ("branch",))
         for path in paths:
             self.assertNotEqual(wt.path2id(path), None)
@@ -56,7 +55,6 @@
         
         self.build_tree(build_paths)
         wt = self.make_branch_and_tree('.')
-        branch = wt.branch
         child_tree = self.make_branch_and_tree('original/child')
         smart_add_tree(wt, (".",))
         for path in paths:
@@ -103,7 +101,6 @@
         from bzrlib.commands import run_bzr
         eq = self.assertEqual
         wt = self.make_branch_and_tree('.')
-        branch = wt.branch
         self.build_tree(['inertiatic/', 'inertiatic/esp'])
         eq(list(wt.unknowns()), ['inertiatic'])
         self.capture('add --dry-run .')
@@ -113,14 +110,12 @@
         """Test smart-adding a file that does not exist."""
         from bzrlib.add import smart_add
         wt = self.make_branch_and_tree('.')
-        branch = wt.branch
         self.assertRaises(NoSuchFile, smart_add_tree, wt, 'non-existant-file')
 
     def test_returns_and_ignores(self):
         """Correctly returns added/ignored files"""
         from bzrlib.commands import run_bzr
         wt = self.make_branch_and_tree('.')
-        branch = wt.branch
         # no files should be ignored by default, so we need to create
         # an ignore rule - we create one for the pyc files, which means
         # CVS should not be ignored.
@@ -142,18 +137,15 @@
         paths = ("original/", "original/file1", "original/file2")
         self.build_tree(paths)
         wt = self.make_branch_and_tree('.')
-        branch = wt.branch
         smart_add_tree(wt, (u".",))
         for path in paths:
             self.assertNotEqual(wt.path2id(path), None)
 
     def test_add_dot_from_subdir(self):
         """Test adding . from a subdir of the tree.""" 
-        from bzrlib.add import smart_add_tree
         paths = ("original/", "original/file1", "original/file2")
         self.build_tree(paths)
         wt = self.make_branch_and_tree('.')
-        branch = wt.branch
         os.chdir("original")
         smart_add_tree(wt, (u".",))
         for path in paths:
@@ -161,20 +153,17 @@
 
     def test_add_tree_from_above_tree(self):
         """Test adding a tree from above the tree.""" 
-        from bzrlib.add import smart_add_tree
         paths = ("original/", "original/file1", "original/file2")
         branch_paths = ("branch/", "branch/original/", "branch/original/file1",
                         "branch/original/file2")
         self.build_tree(branch_paths)
         tree = self.make_branch_and_tree('branch')
-        branch = tree.branch
         smart_add_tree(tree, ("branch",))
         for path in paths:
             self.assertNotEqual(tree.path2id(path), None)
 
     def test_add_above_tree_preserves_tree(self):
         """Test nested trees are not affect by an add above them."""
-        from bzrlib.add import smart_add_tree
         paths = ("original/", "original/file1", "original/file2")
         child_paths = ("path")
         full_child_paths = ("original/child", "original/child/path")
@@ -182,7 +171,6 @@
                        "original/child/", "original/child/path")
         self.build_tree(build_paths)
         tree = self.make_branch_and_tree('.')
-        branch = tree.branch
         child_tree = self.make_branch_and_tree("original/child")
         smart_add_tree(tree, (u".",))
         for path in paths:
@@ -196,16 +184,72 @@
 
     def test_add_paths(self):
         """Test smart-adding a list of paths."""
-        from bzrlib.add import smart_add_tree
         paths = ("file1", "file2")
         self.build_tree(paths)
         wt = self.make_branch_and_tree('.')
-        branch = wt.branch
         smart_add_tree(wt, paths)
         for path in paths:
             self.assertNotEqual(wt.path2id(path), None)
 
 
+class TestAddNonNormalized(TestCaseWithTransport):
+
+    def make(self):
+        try:
+            self.build_tree([u'a\u030a'])
+        except UnicodeError:
+            raise TestSkipped('Filesystem cannot create unicode filenames')
+
+        self.wt = self.make_branch_and_tree('.')
+
+    def test_accessible_explicit(self):
+        self.make()
+        orig = osutils.normalized_filename
+        osutils.normalized_filename = osutils._accessible_normalized_filename
+        try:
+            smart_add_tree(self.wt, [u'a\u030a'])
+            self.assertEqual([(u'\xe5', 'file')],
+                    [(path, ie.kind) for path,ie in 
+                        self.wt.inventory.iter_entries()])
+        finally:
+            osutils.normalized_filename = orig
+
+    def test_accessible_implicit(self):
+        self.make()
+        orig = osutils.normalized_filename
+        osutils.normalized_filename = osutils._accessible_normalized_filename
+        try:
+            smart_add_tree(self.wt, [])
+            self.assertEqual([(u'\xe5', 'file')],
+                    [(path, ie.kind) for path,ie in 
+                        self.wt.inventory.iter_entries()])
+        finally:
+            osutils.normalized_filename = orig
+
+    def test_inaccessible_explicit(self):
+        self.make()
+        orig = osutils.normalized_filename
+        osutils.normalized_filename = osutils._inaccessible_normalized_filename
+        try:
+            self.assertRaises(errors.InvalidNormalization,
+                    smart_add_tree, self.wt, [u'a\u030a'])
+        finally:
+            osutils.normalized_filename = orig
+
+    def test_inaccessible_implicit(self):
+        self.make()
+        orig = osutils.normalized_filename
+        osutils.normalized_filename = osutils._inaccessible_normalized_filename
+        try:
+            # TODO: jam 20060701 In the future, this should probably
+            #       just ignore files that don't fit the normalization
+            #       rules, rather than exploding
+            self.assertRaises(errors.InvalidNormalization,
+                    smart_add_tree, self.wt, [])
+        finally:
+            osutils.normalized_filename = orig
+
+
 class TestAddActions(TestCase):
 
     def test_quiet(self):

=== modified file 'bzrlib/tests/test_urlutils.py'
--- bzrlib/tests/test_urlutils.py	2006-06-30 15:32:22 +0000
+++ bzrlib/tests/test_urlutils.py	2006-07-11 18:34:09 +0000
@@ -17,11 +17,12 @@
 """Tests for the urlutils wrapper."""
 
 import os
+import re
 import sys
 
+from bzrlib import osutils, urlutils
 import bzrlib
 from bzrlib.errors import InvalidURL, InvalidURLJoin
-import bzrlib.urlutils as urlutils
 from bzrlib.tests import TestCaseInTempDir, TestCase, TestSkipped
 
 
@@ -464,3 +465,30 @@
         test('http://host/', 'http://host', 'http://host/')
         #test('.', 'http://host/', 'http://host')
         test('http://host', 'http://host/', 'http://host')
+
+
+class TestCwdToURL(TestCaseInTempDir):
+    """Test that local_path_to_url works base on the cwd"""
+
+    def test_dot(self):
+        # This test will fail if getcwd is not ascii
+        os.mkdir('mytest')
+        os.chdir('mytest')
+
+        url = urlutils.local_path_to_url('.')
+        self.assertEndsWith(url, '/mytest')
+
+    def test_non_ascii(self):
+        try:
+            os.mkdir(u'dod\xe9')
+        except UnicodeError:
+            raise TestSkipped('cannot create unicode directory')
+
+        os.chdir(u'dod\xe9')
+
+        # On Mac OSX this directory is actually: 
+        #   u'/dode\u0301' => '/dode\xcc\x81
+        # but we should normalize it back to 
+        #   u'/dod\xe9' => '/dod\xc3\xa9'
+        url = urlutils.local_path_to_url('.')
+        self.assertEndsWith(url, '/dod%C3%A9')

=== modified file 'bzrlib/tests/workingtree_implementations/test_workingtree.py'
--- bzrlib/tests/workingtree_implementations/test_workingtree.py	2006-07-07 15:22:42 +0000
+++ bzrlib/tests/workingtree_implementations/test_workingtree.py	2006-07-11 18:34:09 +0000
@@ -20,7 +20,7 @@
 import sys
 
 import bzrlib
-from bzrlib import branch, bzrdir, errors, urlutils, workingtree
+from bzrlib import branch, bzrdir, errors, osutils, urlutils, workingtree
 from bzrlib.errors import (NotBranchError, NotVersionedError, 
                            UnsupportedOperation, PathsNotVersionedError)
 from bzrlib.osutils import pathjoin, getcwd, has_symlinks
@@ -632,4 +632,34 @@
         self.assertEqual((u'foo.pyc', 'V', 'file', 'anid'), files[1][:-1])
         self.assertEqual(2, len(files))
 
+    def test_non_normalized_add_accessible(self):
+        try:
+            self.build_tree([u'a\u030a'])
+        except UnicodeError:
+            raise TestSkipped('Filesystem does not support unicode filenames')
+        tree = self.make_branch_and_tree('.')
+        orig = osutils.normalized_filename
+        osutils.normalized_filename = osutils._accessible_normalized_filename
+        try:
+            tree.add([u'a\u030a'])
+            self.assertEqual([(u'\xe5', 'file')],
+                    [(path, ie.kind) for path,ie in 
+                                tree.inventory.iter_entries()])
+        finally:
+            osutils.normalized_filename = orig
+
+    def test_non_normalized_add_inaccessible(self):
+        try:
+            self.build_tree([u'a\u030a'])
+        except UnicodeError:
+            raise TestSkipped('Filesystem does not support unicode filenames')
+        tree = self.make_branch_and_tree('.')
+        orig = osutils.normalized_filename
+        osutils.normalized_filename = osutils._inaccessible_normalized_filename
+        try:
+            self.assertRaises(errors.InvalidNormalization,
+                tree.add, [u'a\u030a'])
+        finally:
+            osutils.normalized_filename = orig
+
 

=== modified file 'bzrlib/workingtree.py'
--- bzrlib/workingtree.py	2006-07-03 04:59:44 +0000
+++ bzrlib/workingtree.py	2006-07-03 18:27:35 +0000
@@ -51,11 +51,10 @@
 from time import time
 import warnings
 
+from bzrlib import bzrdir, errors, osutils, urlutils
 from bzrlib.atomicfile import AtomicFile
 from bzrlib.conflicts import Conflict, ConflictList, CONFLICT_SUFFIXES
-import bzrlib.bzrdir as bzrdir
 from bzrlib.decorators import needs_read_lock, needs_write_lock
-import bzrlib.errors as errors
 from bzrlib.errors import (BzrCheckError,
                            BzrError,
                            ConflictFormatError,
@@ -103,7 +102,6 @@
 from bzrlib.trace import mutter, note
 from bzrlib.transport import get_transport
 from bzrlib.transport.local import LocalTransport
-import bzrlib.urlutils as urlutils
 import bzrlib.ui
 import bzrlib.xml5
 
@@ -356,7 +354,7 @@
         :return: The WorkingTree that contains 'path', and the rest of path
         """
         if path is None:
-            path = os.getcwdu()
+            path = osutils.getcwd()
         control, relpath = bzrdir.BzrDir.open_containing(path)
 
         return control.open_workingtree(), relpath
@@ -768,7 +766,25 @@
                 elif self.is_ignored(fp[1:]):
                     c = 'I'
                 else:
-                    c = '?'
+                    # we may not have found this file, because of a unicode issue
+                    f_norm, can_access = osutils.normalized_filename(f)
+                    if f == f_norm or not can_access:
+                        # No change, so treat this file normally
+                        c = '?'
+                    else:
+                        # this file can be accessed by a normalized path
+                        # check again if it is versioned
+                        # these lines are repeated here for performance
+                        f = f_norm
+                        fp = from_dir_relpath + '/' + f
+                        fap = from_dir_abspath + '/' + f
+                        f_ie = inv.get_child(from_dir_id, f)
+                        if f_ie:
+                            c = 'V'
+                        elif self.is_ignored(fp[1:]):
+                            c = 'I'
+                        else:
+                            c = '?'
 
                 fk = file_kind(fap)
 
@@ -1000,9 +1016,15 @@
 
             fl = []
             for subf in os.listdir(dirabs):
-                if (subf != '.bzr'
-                    and (subf not in dir_entry.children)):
-                    fl.append(subf)
+                if subf == '.bzr':
+                    continue
+                if subf not in dir_entry.children:
+                    subf_norm, can_access = osutils.normalized_filename(subf)
+                    if subf_norm != subf and can_access:
+                        if subf_norm not in dir_entry.children:
+                            fl.append(subf_norm)
+                    else:
+                        fl.append(subf)
             
             fl.sort()
             for subf in fl:

